/*
	Copyright (C) 2003 Alex de Landgraaf <alextreme@xs4all.nl>

	inspired by qp_fswrap in qtparted

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <parted/parted.h>
#include "pm_libparted.h"

#define MKNTFS_PATH "/usr/sbin/mkntfs"
#define MKEXT3_PATH "/sbin/mkfs.ext3"
#define MKXFS_PATH "/sbin/mkfs.xfs"
#define NTFSRESIZE_PATH "/usr/sbin/ntfsresize"

extern  GtkWidget * ProgressWindow;

gboolean ExecuteCommand(gchar *command) {
	gchar *outbuffer;
	gint exitstatus;

	pm_debug("Exec: %s\n",command);
	if (g_spawn_command_line_sync(command,&outbuffer,NULL,&exitstatus,NULL) == FALSE) {
		pm_debug("Error occured executing command\n");
		return FALSE;
	}
	free(outbuffer);
	return TRUE;
}

gboolean pm_wrap_path_available(gchar *path) {
/* somehow statting doesn't always work FIXME
	struct stat *buf;
	if (stat(path,buf) == -1) {
		return FALSE;
	}
	if (buf->st_mode & S_IXUSR) {
		return TRUE;
	}
	return FALSE;
*/
	FILE *fp;
	fp = fopen(path,"r");
	if (fp == NULL) {
		return FALSE;
	}
	fclose(fp);
	return TRUE;
}

gboolean pm_wrap_mkntfs_available() {
	return pm_wrap_path_available(MKNTFS_PATH);
}
gboolean pm_wrap_ntfsresize_available() {
	return pm_wrap_path_available(NTFSRESIZE_PATH);
}
gboolean pm_wrap_mkext3_available() {
	return pm_wrap_path_available(MKEXT3_PATH);
}
gboolean pm_wrap_mkxfs_available() {
	return pm_wrap_path_available(MKXFS_PATH);
}

gchar *do_mkpartfs_retpart(PedDevice **dev, PedPartitionType part_type, 
		      gchar *fs_name,
		      PedSector start, PedSector end) {
	PedDisk *disk;
	PedPartition *part;
	PedConstraint *constraint;
	gchar *partname;
	//char *peek_word;
	PedFileSystemType *fs_type = ped_file_system_type_get(fs_name);

	disk = ped_disk_new(*dev);
	if (!disk) 
		return FALSE;
	constraint = ped_constraint_any(*dev);
	if (!constraint) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	//peek_word = command_line_peek_word();
	part = ped_partition_new(disk,part_type,fs_type,start,end);
	if (!part) {
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!grow_over_small_freespace(&part->geom,disk)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_disk_add_partition(disk,part,constraint)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}

	if (!pm_solution_check_distant(start,end,part->geom.start,part->geom.end)) {
		ped_disk_remove_partition (disk, part);
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	/*
	if (!ped_partition_set_system(part,fs_type)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}*/

	if (!ped_disk_commit(disk)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_constraint_destroy(constraint);
	partname = ped_partition_get_path(part);
	ped_disk_destroy(disk);
	return partname;
}

gboolean do_mkntfs(PedDevice **dev, PedPartitionType part_type, 
		      PedSector start, PedSector end) {

	gchar cmdline[1024];
	gchar* partname = do_mkpartfs_retpart(dev,part_type,"ntfs",start,end);

	sprintf(cmdline,"%s %s",MKNTFS_PATH,partname);
	gboolean ret = ExecuteCommand(cmdline);
	return ret;
}

gboolean do_mkext3(PedDevice **dev, PedPartitionType part_type, 
		      PedSector start, PedSector end) {

	gchar cmdline[1024];
	gchar *partname = do_mkpartfs_retpart(dev,part_type,"ext3",start,end);

	sprintf(cmdline,"%s %s",MKEXT3_PATH,partname);
	display_progress_window();
	gboolean ret = ExecuteCommand(cmdline);
	gtk_widget_hide(ProgressWindow);
	return ret;
}

gboolean do_mkxfs(PedDevice **dev, PedPartitionType part_type, 
		      PedSector start, PedSector end) {

	gchar cmdline[1024];
	gchar *partname = do_mkpartfs_retpart(dev,part_type,"xfs",start,end);

	sprintf(cmdline,"%s -f %s",MKXFS_PATH,partname);
	gboolean ret = ExecuteCommand(cmdline);
	return ret;
}

gboolean pm_wrap_ntfsresize(PedDevice **dev, int part_nr, gchar *path, PedSector start, PedSector end) {
	PedDisk *disk;
	PedPartition *part;
	gchar cmdline[1024];
	disk = ped_disk_new(*dev);
	if (!disk) 
		return FALSE;
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!partition_warn_busy(part,path)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!start || !end) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_disk_destroy(disk);
	/* which partition to resize? close disk! */
	long long sector_nr = end - start;
	sprintf(cmdline,"%s -ff -s %lldM %s",NTFSRESIZE_PATH,(sector_nr * 512 / (1024*1024)),path); // -ff? hope i won't have to use this...
	gboolean ret = ExecuteCommand(cmdline);
	return ret;
}

gboolean do_ntfsresize(PedDevice **dev, int part_nr, PedSector old_end, gchar *path, PedSector start, PedSector new_end) {
	PedDisk *disk;
	PedPartition *part;

	if(!pm_wrap_ntfsresize_available()) {
		pm_debug("ntfsresize not available, aborting");
		return FALSE;
	}

	disk = ped_disk_new(*dev);
	if (!disk) 
		return FALSE;
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!partition_warn_busy(part,path)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!start || !new_end || !old_end) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_disk_destroy(disk);
	if (new_end < old_end) {
		// shrinking
		if (!pm_wrap_ntfsresize(dev,part_nr,path,start,new_end)) {
			pm_debug("ntfsresizing failed, aborting!\n");
			ped_disk_destroy(disk);
			return FALSE;
		}
		pm_do_resize_part_only(dev,part_nr,path,start,new_end);
	}
	if (new_end > old_end) {
		// expanding
		pm_do_resize_part_only(dev,part_nr,path,start,new_end);
		if (!pm_wrap_ntfsresize(dev,part_nr,path,start,new_end)) {
			pm_debug("ntfsresizing failed, aborting!\n");
			pm_do_resize_part_only(dev,part_nr,path,start,old_end);
			ped_disk_destroy(disk);
			return FALSE;
		}
	}
	return TRUE;
}	


	
